-- =====================================================================
-- Copyright  2010-2011 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

library ieee;
use ieee.std_logic_1164.all;   
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all; 
use work.sha3_pkg.all;
use work.groestl_pkg.all;

entity groestl_pq_fx8 is
generic (n:integer := GROESTL_DATA_SIZE_SMALL; rom_style : integer := DISTRIBUTED);
port( 
	clk				: in std_logic;
	rst				: in std_logic;
	p_mode			: in std_logic;	
	sel_rd			: in std_logic_vector(2 downto 0);
	round			: in std_logic_vector(7 downto 0);
	input 			: in std_logic_vector(n/8-1 downto 0);
   	output 			: out std_logic_vector(n/8-1 downto 0));
end groestl_pq_fx8;
  
architecture pq_fx8 of groestl_pq_fx8 is

signal	after_subbyte	: std_logic_vector(n/8-1 downto 0);
signal	addconsa, addconsb,addconsc, addconsd, addconse, addconsf,addconsg, addconsh, addcons		: std_logic_vector(n/8-1 downto 0);
signal	after_preg, after_qreg 		: std_logic_vector(n-1 downto 0);  
signal after_prega, after_pregb, after_pregc, after_pregd,after_prege, after_pregf,after_pregg, after_pregh,
after_qrega, after_qregb, after_qregc, after_qregd,after_qrege, after_qregf,after_qregg, after_qregh :std_logic_vector(n/8-1 downto 0);
signal	after_shiftrow, to_mixbytes, to_mcp, to_mcq		: std_logic_vector(n/8-1 downto 0);
constant zero			: std_logic_vector(n/8-1 downto 0):=(others=>'0'); 
signal wr_p, wr_q	:std_logic;
type matrix is array (0 to n/8-1) of std_logic_vector(7 downto 0);
signal pwire, qwire : matrix;

begin
	
	
gen256: if n=GROESTL_DATA_SIZE_SMALL generate
	
	addconsa(63 downto 56) <= (input(63 downto 56) xor round xor x"00") when p_mode ='1' 
	else (not input(63 downto 56));
	addconsa(55 downto 8) <= input(55 downto 8)  when p_mode ='1' 
	else (not input(55 downto 8)); 
	addconsa(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"ff" xor input(7 downto 0) xor round);	
	
	
	addconsb(63 downto 56) <= (input(63 downto 56) xor round xor x"10") when p_mode ='1' 
	else (not input(63 downto 56));
	addconsb(55 downto 8) <= input(55 downto 8)  when p_mode ='1'
	else (not input(55 downto 8)); 
	addconsb(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"ef" xor input(7 downto 0 ) xor round);
	
	
	addconsc(63 downto 56) <= (input(63 downto 56) xor round xor x"20") when p_mode ='1' 
	else (not input(63 downto 56));
	addconsc(55 downto 8) <= input(55 downto 8)  when p_mode ='1' 
	else(not input(55 downto 8)); 
	addconsc(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"df" xor input(7 downto 0) xor round);
	
	
	addconsd(63 downto 56) <= (input(63 downto 56) xor round xor x"30") when p_mode ='1' 
	else (not input(63 downto 56));
	addconsd(55 downto 8) <= input(55 downto 8)  when p_mode ='1' 
	else(not input(55 downto 8)); 
	addconsd(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"cf" xor input(7 downto 0) xor round);
	
	addconse(63 downto 56) <= (input(63 downto 56) xor round xor x"40") when p_mode ='1' 
	else (not input(63 downto 56));
	addconse(55 downto 8) <= input(55 downto 8)  when p_mode ='1'
	else (not input(55 downto 8)); 
	addconse(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"bf" xor input(7 downto 0) xor round);
	
	
	addconsf(63 downto 56) <= (input(63 downto 56) xor round xor x"50") when p_mode ='1' 
	else (not input(63 downto 56));
	addconsf(55 downto 8) <= input(55 downto 8)  when p_mode ='1'
	else (not input(55 downto 8)); 
	addconsf(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"af" xor input(7 downto 0) xor round);
	
	
	addconsg(63 downto 56) <= (input(63 downto 56) xor round xor x"60") when p_mode ='1' 
	else (not input(63 downto 56));
	addconsg(55 downto 8) <= input(55 downto 8)  when p_mode ='1'
	else (not input(55 downto 8)); 
	addconsg(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"9f" xor input(7 downto 0) xor round);
	
	addconsh(63 downto 56) <= (input(63 downto 56) xor round xor x"70") when p_mode ='1' 
	else (not input(63 downto 56));
	addconsh(55 downto 8) <= input(55 downto 8)  when p_mode ='1'
	else (not input(55 downto 8)); 
	addconsh(7 downto 0) <= input(7 downto 0) when p_mode ='1'
	else (x"8f" xor input(7 downto 0) xor round);

	with sel_rd select	
		addcons <= 	addconsa when "000",
					addconsb when "001",
					addconsc when "010",
					addconsd when "011",
					addconse when "100",
					addconsf when "101",
					addconsg when "110",
					addconsh when others;
					
	
end generate;			  		  


gen512: if n=GROESTL_DATA_SIZE_BIG generate	   
	
	addconsa(127 downto 120) <= (input(127 downto 120) xor round xor x"00") when p_mode ='1' 
	else (not input(127 downto 120));
	addconsa(119 downto 72) <= input(119 downto 72)  when p_mode ='1'
	else (not input(119 downto 72)); 
	addconsa(71 downto 64) <= input(71 downto 64) when p_mode ='1' 
	else (x"ff" xor input(71 downto 64) xor round); 
	addconsa(63 downto 56) <= (input(63 downto 56) xor round xor x"10") when p_mode ='1'
	else (not input(63 downto 56));
	addconsa(55 downto 8) <= input(55 downto 8)  when p_mode ='1'
	else (not input(55 downto 8)); 
	addconsa(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"ef" xor input(7 downto 0) xor round); 
	
	
	addconsb(127 downto 120) <= (input(127 downto 120) xor round xor x"20") when p_mode ='1' 
	else (not input(127 downto 120));
	addconsb(119 downto 72) <= input(119 downto 72)  when p_mode ='1'
	else (not input(119 downto 72)); 
	addconsb(71 downto 64) <= input(71 downto 64) when p_mode ='1' 
	else (x"df" xor input(71 downto 64) xor round);  
	addconsb(63 downto 56) <= (input(63 downto 56) xor round xor x"30") when p_mode ='1' 
	else (not input(63 downto 56));
	addconsb(55 downto 8) <= input(55 downto 8)  when p_mode ='1'
	else (not input(55 downto 8)); 
	addconsb(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"cf" xor input(7 downto 0) xor round);
	
	
	
	addconsc(127 downto 120) <= (input(127 downto 120) xor round xor x"40") when p_mode ='1' 
	else (not input(127 downto 120));
	addconsc(119 downto 72) <= input(119 downto 72)  when p_mode ='1'
	else (not input(119 downto 72)); 
	addconsc(71 downto 64) <= input(71 downto 64) when p_mode ='1' 
	else (x"bf" xor input(71 downto 64) xor round);
	addconsc(63 downto 56) <= (input(63 downto 56) xor round xor x"50") when p_mode ='1' 
	else (not input(63 downto 56));
	addconsc(55 downto 8) <= input(55 downto 8)  when p_mode ='1'
	else (not input(55 downto 8)); 
	addconsc(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"af" xor input(7 downto 0) xor round);
	
	
	addconsd(127 downto 120) <= (input(127 downto 120) xor round xor x"60") when p_mode ='1' 
	else (not input(127 downto 120));
	addconsd(119 downto 72) <= input(119 downto 72)  when p_mode ='1'
	else (not input(119 downto 72)); 
	addconsd(71 downto 64) <= input(71 downto 64) when p_mode ='1' 
	else (x"9f" xor input(71 downto 64) xor round);
	addconsd(63 downto 56) <= (input(63 downto 56) xor round xor x"70") when p_mode ='1' 
	else (not input(63 downto 56));
	addconsd(55 downto 8) <= input(55 downto 8)  when p_mode ='1'
	else (not input(55 downto 8)); 
	addconsd(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"8f" xor input(7 downto 0) xor round);
	
	addconse(127 downto 120) <= (input(127 downto 120) xor round xor x"80") when p_mode ='1' 
	else (not input(127 downto 120));
	addconse(119 downto 72) <= input(119 downto 72)  when p_mode ='1'
	else (not input(119 downto 72)); 
	addconse(71 downto 64) <= input(71 downto 64) when p_mode ='1' 
	else (x"7f" xor input(71 downto 64) xor round);
	addconse(63 downto 56) <= (input(63 downto 56) xor round xor x"90") when p_mode ='1' 
	else (not input(63 downto 56));
	addconse(55 downto 8) <= input(55 downto 8)  when p_mode ='1'
	else (not input(55 downto 8)); 
	addconse(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"6f" xor input(7 downto 0) xor round);
	
	
	
	addconsf(127 downto 120) <= (input(127 downto 120) xor round xor x"a0") when p_mode ='1' 
	else (not input(127 downto 120));
	addconsf(119 downto 72) <= input(119 downto 72)  when p_mode ='1'
	else (not input(119 downto 72)); 
	addconsf(71 downto 64) <= input(71 downto 64) when p_mode ='1' 
	else (x"5f" xor input(71 downto 64) xor round);
	addconsf(63 downto 56) <= (input(63 downto 56) xor round xor x"b0") when p_mode ='1' 
	else (not input(63 downto 56));
	addconsf(55 downto 8) <= input(55 downto 8)  when p_mode ='1'
	else (not input(55 downto 8)); 
	addconsf(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"4f" xor input(7 downto 0) xor round);
	
	
	
	addconsg(127 downto 120) <= (input(127 downto 120) xor round xor x"c0") when p_mode ='1' 
	else (not input(127 downto 120));
	addconsg(119 downto 72) <= input(119 downto 72)  when p_mode ='1'
	else (not input(119 downto 72)); 
	addconsg(71 downto 64) <= input(71 downto 64) when p_mode ='1' 
	else (x"3f" xor input(71 downto 64) xor round);
	addconsg(63 downto 56) <= (input(63 downto 56) xor round xor x"d0") when p_mode ='1' 
	else (not input(63 downto 56));
	addconsg(55 downto 8) <= input(55 downto 8)  when p_mode ='1'
	else (not input(55 downto 8)); 
	addconsg(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"2f" xor input(7 downto 0) xor round);
	
	
	addconsh(127 downto 120) <= (input(127 downto 120) xor round xor x"e0") when p_mode ='1' 
	else (not input(127 downto 120));
	addconsh(119 downto 72) <= input(119 downto 72)  when p_mode ='1'
	else (not input(119 downto 72)); 
	addconsh(71 downto 64) <= input(71 downto 64) when p_mode ='1' 
	else (x"1f" xor input(71 downto 64) xor round);
	addconsh(63 downto 56) <= (input(63 downto 56) xor round xor x"f0") when p_mode ='1' 
	else (not input(63 downto 56));
	addconsh(55 downto 8) <= input(55 downto 8)  when p_mode ='1'
	else (not input(55 downto 8)); 
	addconsh(7 downto 0) <= input(7 downto 0) when p_mode ='1' 
	else (x"0f" xor input(7 downto 0) xor round);

	with sel_rd select	
		addcons <= 	addconsa when "000",
					addconsb when "001",
					addconsc when "010",
					addconsd when "011",
					addconse when "100",
					addconsf when "101",
					addconsg when "110",
					addconsh when others;
		
end generate;
	
	
	sbox_gen: for i in 0 to n/(8*AES_SBOX_SIZE) - 1  generate
	sbox	: aes_sbox 	generic map (rom_style=>rom_style)
			port map (	 
				input=>addcons(AES_SBOX_SIZE*i + 7 downto AES_SBOX_SIZE*i), 
				output=>after_subbyte(AES_SBOX_SIZE*i+7 downto AES_SBOX_SIZE*i));	
	end generate;	
	
	wr_p <= '1' when p_mode ='1' else '0';
	wr_q <= '1' when p_mode ='0' else '0';	
	
	pl_reg01		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_p, input=>after_subbyte, output=>after_prega);
	pl_reg02		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_p, input=>after_prega, output=>after_pregb);
	pl_reg03		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_p, input=>after_pregb, output=>after_pregc);
	pl_reg04		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_p, input=>after_pregc, output=>after_pregd);
	pl_reg05		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_p, input=>after_pregd, output=>after_prege);
	pl_reg06		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_p, input=>after_prege, output=>after_pregf);
	pl_reg07		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_p, input=>after_pregf, output=>after_pregg);
	pl_reg08		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_p, input=>after_pregg, output=>after_pregh);
	
	
	pl_reg11		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_q, input=>after_subbyte, output=>after_qrega);
	pl_reg12		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_q, input=>after_qrega, output=>after_qregb);
	pl_reg13		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_q, input=>after_qregb, output=>after_qregc);
	pl_reg14		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_q, input=>after_qregc, output=>after_qregd);
	pl_reg15		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_q, input=>after_qregd, output=>after_qrege);
	pl_reg16		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_q, input=>after_qrege, output=>after_qregf);
	pl_reg17		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_q, input=>after_qregf, output=>after_qregg);
	pl_reg18		: regn generic map (n=>n/8, init=>zero) port map (clk=>clk, rst=>rst, en=>wr_q, input=>after_qregg, output=>after_qregh);
	
	
	after_preg <= after_pregh & after_pregg & after_pregf & after_prege & after_pregd & after_pregc & after_pregb & after_prega;
	after_qreg <= after_qregh & after_qregg & after_qregf & after_qrege & after_qregd & after_qregc & after_qregb & after_qrega;
	
to_matrix: for i in 0 to n/8-1 generate
	pwire(i) <= after_preg((n/8-i)*8-1 downto (n/8-i-1)*8);
	qwire(i) <= after_qreg((n/8-i)*8-1 downto (n/8-i-1)*8);
end generate;
	


mc256: if n=GROESTL_DATA_SIZE_SMALL generate	

	to_mcp <= 	pwire(0) & pwire(9) & pwire(18) & pwire(27) &
				pwire(36) & pwire(45) & pwire(54) & pwire(63)  when sel_rd="000" else
				pwire(8) & pwire(17) & pwire(26) & pwire(35) &
				pwire(44) & pwire(53) &	pwire(62) & pwire(7) when sel_rd="001" else
				pwire(16) & pwire(25) & pwire(34) & pwire(43) &
				pwire(52) & pwire(61) & pwire(6) & pwire(15)  when sel_rd="010" else
				pwire(24) & pwire(33) & pwire(42) & pwire(51) &
				pwire(60) & pwire(5) & pwire(14) & pwire(23) when sel_rd="011" else 
				pwire(32) & pwire(41) & pwire(50) & pwire(59) &
				pwire(4) & pwire(13) & pwire(22) & pwire(31)  when sel_rd="100" else
				pwire(40) & pwire(49) & pwire(58) & pwire(3) &
				pwire(12) & pwire(21) &	pwire(30) & pwire(39) when sel_rd="101" else
				pwire(48) & pwire(57) & pwire(2) & pwire(11) &
				pwire(20) & pwire(29) & pwire(38) & pwire(47) when sel_rd="110" else
				pwire(56) & pwire(1) & pwire(10) & pwire(19) &
				pwire(28) & pwire(37) & pwire(46) & pwire(55) when sel_rd="111";	

	to_mcq <= 	qwire(8) & qwire(25) & qwire(42) & qwire(59) &
				qwire(4) & qwire(21) & qwire(38) & qwire(55)  when sel_rd="000" else
				qwire(16) & qwire(33) & qwire(50) & qwire(3) &
				qwire(12) & qwire(29) &	qwire(46) & qwire(63) when sel_rd="001" else
				qwire(24) & qwire(41) & qwire(58) & qwire(11) &
				qwire(20) & qwire(37) & qwire(54) & qwire(7)  when sel_rd="010" else
				qwire(32) & qwire(49) & qwire(2) & qwire(19) &
				qwire(28) & qwire(45) & qwire(62) & qwire(15) when sel_rd="011" else 
				qwire(40) & qwire(57) & qwire(10) & qwire(27) &
				qwire(36) & qwire(53) & qwire(6) & qwire(23)  when sel_rd="100" else
				qwire(48) & qwire(1) & qwire(18) & qwire(35) &
				qwire(44) & qwire(61) &	qwire(14) & qwire(31) when sel_rd="101" else
				qwire(56) & qwire(9) & qwire(26) & qwire(43) &
				qwire(52) & qwire(5) & qwire(22) & qwire(39)  when sel_rd="110" else
				qwire(0) & qwire(17) & qwire(34) & qwire(51) &
				qwire(60) & qwire(13) & qwire(30) & qwire(47) when sel_rd="111";				
			
	to_mixbytes <= to_mcp when p_mode='0' else to_mcq;

	mc			: entity work.groestl_mixbytes(groestl_mixbytes) generic map (n=>n/8) port map (input=>to_mixbytes,  output=>output);	
end generate;

mc512: if n=GROESTL_DATA_SIZE_BIG generate	

	to_mcp <= 	pwire(0) & pwire(9) & pwire(18) & pwire(27) &
				pwire(36) & pwire(45) & pwire(54) & pwire(95) &
				pwire(8) & pwire(17) & pwire(26) & pwire(35) &
				pwire(44) & pwire(53) &	pwire(62) & pwire(103)  when sel_rd="000" else 
				pwire(16) & pwire(25) & pwire(34) & pwire(43) &
				pwire(52) & pwire(61) & pwire(70) & pwire(111) &
				pwire(24) & pwire(33) & pwire(42) & pwire(51) &
				pwire(60) & pwire(69) & pwire(78) & pwire(119) when sel_rd="001" else 
				pwire(32) & pwire(41) & pwire(50) & pwire(59) &
				pwire(68) & pwire(77) & pwire(86) & pwire(127) &
				pwire(40) & pwire(49) & pwire(58) & pwire(67) &
				pwire(76) & pwire(85) &	pwire(94) & pwire(7)  when sel_rd="010" else 
				pwire(48) & pwire(57) & pwire(66) & pwire(75) &
				pwire(84) & pwire(93) & pwire(102) & pwire(15) &
				pwire(56) & pwire(65) & pwire(74) & pwire(83) &
				pwire(92) & pwire(101) & pwire(110) & pwire(23)when sel_rd="011" else
				pwire(64) & pwire(73) & pwire(82) & pwire(91) &
				pwire(100) & pwire(109) & pwire(118) & pwire(31) &
				pwire(72) & pwire(81) & pwire(90) & pwire(99) &
				pwire(108) & pwire(117) &	pwire(126) & pwire(39) when sel_rd="100" else 
				pwire(80) & pwire(89) & pwire(98) & pwire(107) &
				pwire(116) & pwire(125) & pwire(6) & pwire(47) &
				pwire(88) & pwire(97) & pwire(106) & pwire(115) &
				pwire(124) & pwire(5) & pwire(14) & pwire(55) when sel_rd="101" else
				pwire(96) & pwire(105) & pwire(114) & pwire(123) &
				pwire(4) & pwire(13) & pwire(22) & pwire(63) &
				pwire(104) & pwire(113) & pwire(122) & pwire(3) &
				pwire(12) & pwire(21) &	pwire(30) & pwire(71)  when sel_rd="110" else 
				pwire(112) & pwire(121) & pwire(2) & pwire(11) &
				pwire(20) & pwire(29) & pwire(38) & pwire(79) &
				pwire(120) & pwire(1) & pwire(10) & pwire(19) &
				pwire(28) & pwire(37) & pwire(46) & pwire(87)when sel_rd="111";				

	to_mcq <= 	qwire(8) & qwire(25) & qwire(42) & qwire(91) &
				qwire(4) & qwire(21) & qwire(38) & qwire(55) &
				qwire(16) & qwire(33) & qwire(50) & qwire(99) &
				qwire(12) & qwire(29) &	qwire(46) & qwire(63) when sel_rd="000" else
				qwire(24) & qwire(41) & qwire(58) & qwire(107) &
				qwire(20) & qwire(37) & qwire(54) & qwire(71) &
				qwire(32) & qwire(49) & qwire(66) & qwire(115) &
				qwire(28) & qwire(45) & qwire(62) & qwire(79)  when sel_rd="001" else  
				qwire(40) & qwire(57) & qwire(74) & qwire(123) &
				qwire(36) & qwire(53) & qwire(70) & qwire(87) &
				qwire(48) & qwire(65) & qwire(82) & qwire(3) &
				qwire(44) & qwire(61) &	qwire(78) & qwire(95) when sel_rd="010" else
				qwire(56) & qwire(73) & qwire(90) & qwire(11) &
				qwire(52) & qwire(69) & qwire(86) & qwire(103) &
				qwire(64) & qwire(81) & qwire(98) & qwire(19) &
				qwire(60) & qwire(77) & qwire(94) & qwire(111)when sel_rd="011" else		
				qwire(72) & qwire(89) & qwire(106) & qwire(27) &
				qwire(68) & qwire(85) & qwire(102) & qwire(119) &
				qwire(80) & qwire(97) & qwire(114) & qwire(35) &
				qwire(76) & qwire(93) &	qwire(110) & qwire(127) when sel_rd="100" else
				qwire(88) & qwire(105) & qwire(122) & qwire(43) &
				qwire(84) & qwire(101) & qwire(118) & qwire(7) &
				qwire(96) & qwire(113) & qwire(2) & qwire(51) &
				qwire(92) & qwire(109) & qwire(126) & qwire(15) when sel_rd="101" else  
				qwire(104) & qwire(121) & qwire(10) & qwire(59) &
				qwire(100) & qwire(117) & qwire(6) & qwire(23) &
				qwire(112) & qwire(1) & qwire(18) & qwire(67) &
				qwire(108) & qwire(125) &	qwire(14) & qwire(31) when sel_rd="110" else
				qwire(120) & qwire(9) & qwire(26) & qwire(75) &
				qwire(116) & qwire(5) & qwire(22) & qwire(39) &
				qwire(0) & qwire(17) & qwire(34) & qwire(83) &
				qwire(124) & qwire(13) & qwire(30) & qwire(47)when sel_rd="111";
				
				
	to_mixbytes <= to_mcp when p_mode='0' else to_mcq;
	
	mc_left			: entity work.groestl_mixbytes(groestl_mixbytes) generic map (n=>n/8) port map (input=>to_mixbytes,  output=>output);	
	--mc_right		: entity work.groestl_mixbytes(groestl_mixbytes) port map (input=>after_shiftrow(511 downto 0),  output=>output(511 downto 0));	

end generate;

	
end pq_fx8; 			 	   

