-- =====================================================================
-- Copyright  2010-2011 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

library ieee;
use ieee.std_logic_1164.all;
use work.sha3_pkg.all;
use work.groestl_pkg.all;	 

-- Possible generics values: 
--      hs = {HASH_SIZE_256, HASH_SIZE_512}, 
--      rom_style = {DISTRIBUTED, COMBINATIONAL}	
--      version = {SHA3_ROUND2, SHA3_ROUND3}
--      ff = {2, 4, 8}  -- Folding factor

-- All combinations are allowed				

entity groestl_top_pq_folded is		
	generic ( 
		version		:integer	:= SHA3_ROUND3;
		rom_style	:integer	:= DISTRIBUTED;
		hs 			:integer 	:= HASH_SIZE_256;
		ff			:integer	:= 2); 
	port (		
		rst 		: in std_logic;
		clk 		: in std_logic;
		src_ready 	: in std_logic;
		src_read  	: out std_logic;
		dst_ready 	: in std_logic;
		dst_write 	: out std_logic;		
		din			: in std_logic_vector(w-1 downto 0);
		dout		: out std_logic_vector(w-1 downto 0)
	);	   
end groestl_top_pq_folded;


architecture struct of groestl_top_pq_folded is  



begin

        
    fx2_core: if ff=2 generate
        fx2_gen:entity work.groestl_top_pq_fx2(struct)
        generic map ( hs => hs, ff=>ff, version=>version, rom_style=>rom_style)
        port map (rst => rst, clk=> clk, din => din, src_read => src_read, src_ready => src_ready, dout => dout, dst_write => dst_write, dst_ready => dst_ready); 
    end generate;

    fx4_core: if ff=4 generate
        fx4_gen:entity work.groestl_top_pq_fx4(struct)
        generic map ( hs => hs, ff=>ff, version=>version, rom_style=>rom_style)
        port map (rst => rst, clk=> clk, din => din, src_read => src_read, src_ready => src_ready, dout => dout, dst_write => dst_write, dst_ready => dst_ready); 
    end generate;

    fx8_core: if ff=8 generate
        fx8_gen:entity work.groestl_top_pq_fx8(struct)
        generic map ( hs => hs, ff=>ff, version=>version, rom_style=>rom_style)
        port map (rst => rst, clk=> clk, din => din, src_read => src_read, src_ready => src_ready, dout => dout, dst_write => dst_write, dst_ready => dst_ready); 
    end generate;

		
end struct;
	
	